<?php
  /**
   * View requests & requests specifics.
   * @author Jason Grafinger
   */

  session_start();
  require("./include/include.php");

  connect($master);

  //This is a secure page, so check user's authentication.
  if(!isset($_SESSION['userType']))
  {
    //Unauthenticated user, redirect.
    header("Location: " . $master['systemRoot'] . "?error=badPage");
  }

  //Display List
  if(!isset($_GET['requestID']))
  {
    //Array that maps $_GET['orderBy'] to MySQL columns
    $sortByMap = array(
      "id" => "requests.requestID",
      "name" => "users.lastName",
      "date" => "requests.dateIssued",
      "type" => "requests.requestTypeID",
      "status" => "requests.requestStatusID");

    //Strings to be added to various HTML strings using sprintf()
    $sortByHTML = "<a href=\"{$_SERVER['PHP_SELF']}?sortby=%s&amp;dir=%s\">" .
      "%s</a>";
    $adminExtraCols = "";
    if($_SESSION['userType'] == "admin")
    {
      //Admins need to see WHO a request is from
      $adminExtraCols = sprintf("\n          <th>Name %s/%s</th>\n",
        sprintf($sortByHTML,"name","ASC","^"),
        sprintf($sortByHTML,"name","DESC", "v"));
    }
    $tableHTML = sprintf("      <table width=\"100%s\" border=\"1\">
        <tr>
          <th>Request ID %s/%s</th>%s
          <th>Date Issued %s/%s</th>
          <th>Type %s/%s</th>
          <th>Status %s/%s</th>
        </tr>\n",
      "%",
      sprintf($sortByHTML,"id","ASC","^"),
      sprintf($sortByHTML,"id","DESC","v"),
      $adminExtraCols,
      sprintf($sortByHTML,"date","ASC","^"),
      sprintf($sortByHTML,"date","DESC","v"),
      sprintf($sortByHTML,"type","ASC","^"),
      sprintf($sortByHTML,"type","DESC","v"),
      sprintf($sortByHTML,"status","ASC","^"),
      sprintf($sortByHTML,"status","DESC","v"));


    //Prepare the query
    if(isset($_GET['sortby']) && isset($_GET['dir']) &&
      isset($sortByMap[$_GET['sortby']]))
    {
      //User specified order
      $orderByString = $sortByMap[$_GET['sortby']];
      $orderByDirectionString = mysql_real_escape_string($_GET['dir']);
    }
    else
    {
      //Default order
      $orderByString = "requests.requestID";
      $orderByDirectionString = "ASC";
    }
    $query = sprintf("SELECT requests.*,requestSubTypes.requestSubTypeName," . 
      "requestTypes.requestTypeName,users.firstName,users.lastName, " .
      "requestStatus.requestStatusName " .
      "FROM requests INNER JOIN requestTypes ON requests.requestTypeID=" .
      "requestTypes.requestTypeID " .
      "INNER JOIN users ON requests.userID=users.userID " .
      "INNER JOIN requestStatus ON requests.requestStatusID=requestStatus." .
      "requestStatusID " .
      "LEFT OUTER JOIN requestSubTypes ON requests." .
      "requestSubTypeID=requestSubTypes.requestSubTypeID ORDER BY %s %s",
      $orderByString,$orderByDirectionString);

    //Make the query
    $result = mysql_query($query) OR die("Query Error: " . mysql_error());

    //Display list of requests.
    $nonUserCount = 0;
    $totalRowsPrinted = 0;
    $nonUserRowHTML = "        <tr><td colspan=\"4\" style=\"text-align:center;\">=== %d " .
      "requests from other students ===</td></tr>\n";

    $adminExtraStandardCols = ($_SESSION['userType'] == "admin") ?
      "\n          <td>%s</td>\n" :
      "";
    $standardRowHTML = "        <tr>
          <td>%s</td>" . $adminExtraStandardCols . "
          <td>%s</td>
          <td>%s</td>
          <td>%s</td>
        </tr>\n";

    while($row = mysql_fetch_assoc($result))
    {
      if($_SESSION['userType'] != "admin")
      {
        //STD USER... Only display user's rows
        if($row['userID'] == $_SESSION['userID'])
        {
          //Request belongs to the user
          if($nonUserCount > 0)
          {
            $tableHTML .= sprintf($nonUserRowHTML,$nonUserCount);
          }
          $tableHTML .= sprintf($standardRowHTML,
            sprintf("%s [<a href=\"{$_SERVER['PHP_SELF']}?requestID=%s\">View</a>]",
              $row['requestID'],$row['requestID']),
            $row['dateIssued'],
            $row['requestTypeName'],
            $row['requestStatusName']);

          $nonUserCount = 0;
        }
        else
        {
          //Dont print anything, but keep track of the row
          $nonUserCount++;
        }
      }
      else
      {
        //ADMIN... Display all rows
        $tableHTML .= sprintf($standardRowHTML,
          sprintf("%s [<a href=\"{$_SERVER['PHP_SELF']}?requestID=%s\">View</a>]",
            $row['requestID'],$row['requestID']),
          $row['firstName'] . " " . $row['lastName'],
          $row['dateIssued'],
          $row['requestTypeName'],
          $row['requestStatusName']);
      }

      $totalRowsPrinted++;
    }
    if($nonUserCount > 0)
    {
      //If there are requests AFTER the last request
      // belonging to the user, print the $nonUserRowHTML.
      // (Works for NO requests belonging to the user as well)
      $tableHTML .= sprintf($nonUserRowHTML,$nonUserCount);
    }
    $tableHTML .= "      </table>\n";

    //Write the page
    writeTop("View All Requests",false,$master,$_GET['error']);
    writeNav($_SESSION['userType']);
    echo "      <hr />\n";
    echo $tableHTML;
  }

  //$_GET['requestID'] IS set
  else
  {
    //Display specific request.
    if(is_numeric($_GET['requestID']))
    {
      $query = "SELECT requests.requestID," .
        "users.firstName, users.lastName," .
        "requestTypes.requestTypeName as 'Request Type'," .
        "requestSubTypes.requestSubTypeName as 'Request Subtype'," .
        "requests.dateIssued as 'Date Issued'," .
        "requests.deadline as 'Deadline'," .
        "requests.requestedCompletionDate as 'Requested Completion Date'," .
        "requestStatus.requestStatusName as 'Status'," .
        "requests.url1,requests.url2,requests.url3," .
        "requests.notes as 'Notes'" .
        "FROM requests " .
        "INNER JOIN requestTypes ON requests.requestTypeID=" .
        "requestTypes.requestTypeID " .
        "INNER JOIN users ON requests.userID=users.userID " .
        "INNER JOIN requestStatus ON requests.requestStatusID=" .
        "requestStatus.requestStatusID " .
        "LEFT OUTER JOIN requestSubTypes ON requests.requestSubTypeID=" .
        "requestSubTypes.requestSubTypeID WHERE requests.requestID=" .
        mysql_real_escape_string($_GET['requestID']) . "%s";

      //Add special conditions for std-users to the query.
      // Std-users need to check against their userID for
      // security purposes.
      $queryConditions = "";
      if($_SESSION['userType'] == "std")
      {
        $queryConditions = " AND requests.userID={$_SESSION['userID']}";
      }
      $query = sprintf($query,$queryConditions);

      $result = mysql_query($query) OR die("Could not query the database.");

      if(mysql_num_rows($result) == 1)
      {
        //Got a result, display in a table.
        $row = mysql_fetch_assoc($result);

        $requestInfoHTML = "      <table border=\"1\">
        <tr>
          <th>Field</th>
          <th>Value</th>
        </tr>\n";

        $tableData = array();

        //Admins need to see WHO the request is for!
        if($_SESSION['userType'] == "admin")
        {
          $tableData['Name'] = $row['firstName'] . " " . $row['lastName'];
        }

        //Add all request info _keys_ to an array
        foreach(array_keys($row) as $requestInfoName)
        {
          //Only put things that have data
          // Also, disregard firstName and lastName, they're
          // already added
          if(isset($row[$requestInfoName]) &&
            $requestInfoName != "firstName" &&
            $requestInfoName != "lastName")
          {
            $tableData[$requestInfoName] = $row[$requestInfoName];
          }
        }

        //Now add relavent fields to the tableHTML
        foreach(array_keys($tableData) as $field)
        {
          $insertData = $tableData[$field];

          //Requirement: Notes are in a <pre> tag
          if($field == "Notes")
          {
            $insertData = sprintf("\n            <pre>" .
              "\n%s\n" .
              "            </pre>\n          ",
              $insertData);
          }

          //URLs need to be linked
          if(strstr($field,"url"))
          {
            $insertData = sprintf("<a target=\"_blank\" href=\"%s\">%s</a>",
              $insertData,$insertData);
          }

          //Add the row
          $requestInfoHTML .= "        <tr>
          <td>{$field}</td>
          <td>{$insertData}</td>
        </tr>\n";
        }

        $requestInfoHTML .= "      </table>
      <p>
        <a href=\"editrequest.php?requestID={$_GET['requestID']}\">Edit Request</a> |
        <a href=\"deleterequest.php?requestID={$_GET['requestID']}\">Delete Request</a>
      </p>\n";

        writeTop("View Request Info For ID: {$_GET['requestID']}",false,$master,$_GET['error']);
        writeNav($_SESSION['userType']);
        echo "      <hr />\n";
        echo $requestInfoHTML;
      }
      else
      {
        //Invalid requestID
        header("Location: " . $_SERVER['PHP_SELF'] . "?error=req1");
      }
    }
    else
    {
      //Illegal argument
      header("Location: " . $_SERVER['PHP_SELF'] . "?error=badPage");
    }
  }

  writeBottom();
?>
